package com.cusc.nirvana.user.rnr.openapi.config;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.springframework.cloud.client.loadbalancer.LoadBalanced;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.ClientHttpRequestFactory;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;

/**
 * RestTemplate对象
 *
 * @author yubo
 * @since 2022-04-18 09:54
 */
@Configuration
@Slf4j
public class RestTemplateRnrConfig {

    @Bean(name = "eRestTemplate")
    @LoadBalanced
    public RestTemplate restTemplate() {
        return getRestTemplateRnr();
    }

    @Bean
    RestTemplate nonLoadBalanced() {
        return getRestTemplateRnr();
    }

    public RestTemplate getRestTemplateRnr() {
        RestTemplate restTemplate = new RestTemplate(httpRequestFactoryRnr());
        List<ClientHttpRequestInterceptor> list = new ArrayList<>();
        list.add((request, body, execution) -> {
            long startTime = System.currentTimeMillis();
            log.info("getRestTemplateRnr request url:{} , params:{}", request.getURI(), new String(body));
            ClientHttpResponse ret = execution.execute(request, body);
            log.info("getRestTemplateRnr response url:{} , result:{} , cost:{} ms", request.getURI(),
                    JSON.toJSONString(ret),
                    System.currentTimeMillis() - startTime);
            return ret;
        });
        restTemplate.setInterceptors(list);
        return restTemplate;
    }


    private ClientHttpRequestFactory httpRequestFactoryRnr() {
        return new HttpComponentsClientHttpRequestFactory(httpClientRnr());
    }

    private HttpClient httpClientRnr() {
        Registry<ConnectionSocketFactory> registry = RegistryBuilder.<ConnectionSocketFactory>create()
                .register("http", PlainConnectionSocketFactory.getSocketFactory())
                .register("https", SSLConnectionSocketFactory.getSocketFactory())
                .build();
        PoolingHttpClientConnectionManager connectionManager = new PoolingHttpClientConnectionManager(registry);
        //设置整个连接池最大连接数
        connectionManager.setMaxTotal(100);
        //路由是对maxTotal的细分
        connectionManager.setDefaultMaxPerRoute(100);
        RequestConfig requestConfig = RequestConfig.custom()
                .setSocketTimeout(5000)  //返回数据的超时时间
                .setConnectTimeout(3000) //连接上服务器的超时时间
                .setConnectionRequestTimeout(1000) //从连接池中获取连接的超时时间
                .build();
        return HttpClientBuilder.create()
                .setDefaultRequestConfig(requestConfig)
                .setConnectionManager(connectionManager)
                .build();
    }
}
