package com.cusc.nirvana.user.rnr.openapi.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrUnboundClient;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.VehicleUnbindDTO;
import com.cusc.nirvana.user.rnr.openapi.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.openapi.dto.RnrUnboundReqDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.SecondHandUnbindRequestDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.VehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.groups.VehicleCardRnrGroup;
import com.cusc.nirvana.user.rnr.openapi.service.ICardUnBindService;
import com.cusc.nirvana.user.rnr.openapi.util.UserSubjectUtil;
import com.cusc.nirvana.user.util.CuscStringUtils;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Description: 实名解绑控制层
 * <br />
 * CreateDate 2022-04-16 15:16:55
 *
 * @author yuyi
 **/
@RestController
@RequestMapping("/rnAuth/unbind")
public class OpenApiCardUnBindController {

    @Autowired
    private MgRnrCardInfoClient rnrCardInfoClient;

    @Autowired
    private MgRnrUnboundClient unboundClient;

    @Autowired
    ICardUnBindService cardUnBindService;

    @Autowired
    private MgRnrInfoClient rnrInfoClient;

    @Autowired
    private MgRnrCardInfoClient mgRnrCardInfoClient;


    @ApiOperation(value = "原车主解绑即一手车解绑操作", notes = "原车主解绑即一手车解绑操作")
    @PostMapping("/firstHandCar")
    public Response firstHandCar(@Validated(VehicleCardRnrGroup.Submit.class) @RequestBody VehicleCardRnrDTO bean) {
        String rnrId = getRnrIdByVinIccid(bean.getVin(), null);
        if (StringUtils.isEmpty(rnrId)) {
            return Response.createError("根据vin没有查询到任何已经实名的卡");
        }
        bean.setRnrId(rnrId);
        //todo api层 解绑 短信 回调流程 待确认开发   是否轮询结果 然后给与回调
        Response response = cardUnBindService.originalOwner(bean);
        if (response.isSuccess()) {
            return Response.createSuccess("解绑请求已受理,请查收解绑短信");
        } else {
            return response;
        }
    }

    @ApiOperation(value = "二手车现车主解绑-自然人", notes = "二手车现车主解绑-自然人")
    @PostMapping("/secondHandCar")
    public Response secondHandCar(@Valid @RequestBody SecondHandUnbindRequestDTO bean) {
        //todo api层 解绑 短信 回调流程 待确认开发  是否轮询结果 然后给与回调
        return cardUnBindService.secondHandPersonalUnbind(bean);
    }


    @ApiOperation(value = "发送短信", notes = "发送短信")
    @PostMapping("/sendMessage")
    public Response sendMessage(@RequestBody VehicleCardRnrDTO bean) {
        return cardUnBindService.sendMessage(bean);
    }


    @ApiOperation(value = "轮询查询工单状态", notes = "轮询查询工单状态")
    @PostMapping("/checkOrderStatus")
    public Response checkOrderStatus(@RequestBody RnrOrderDTO bean) {
        return cardUnBindService.checkOrderStatus(bean);
    }


    @PostMapping("/byIccid")
    @ApiOperation(value = "解绑", notes = "解绑")
    public Response unbound(@Valid @RequestBody RnrUnboundReqDTO bean) {

//        String rnrId = getRnrIdByVinIccid(null, bean.getIccid());
//        if (StringUtils.isEmpty(rnrId)) {
//            return Response.createError("根据iccid没有查询到任何已经实名的卡");
//        }

        //验证卡的状态
        MgRnrCardInfoDTO card = new MgRnrCardInfoDTO();
        card.setIccid(bean.getIccid());
        card.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<MgRnrCardInfoDTO> cardResp = rnrCardInfoClient.getOneByIccid(card);
        if (cardResp == null || !cardResp.isSuccess() || cardResp.getData() == null) {
            return Response.createError(ResponseCode.ICCID_INVALID.getMsg(), ResponseCode.ICCID_INVALID.getCode());
        }
//        //判断输入的rnrid 和 iccid查询出来的 rnrId是否一致
//        if (!cardResp.getData().getRnrId().equals(rnrId)) {
//            return Response.createError(ResponseCode.INVALID_DATA.getMsg(), ResponseCode.INVALID_DATA.getCode());
//        }

        MgRnrInfoDTO info = new MgRnrInfoDTO();
        info.setUuid(cardResp.getData().getRnrId());
        info.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<MgRnrInfoDTO> infoResp = rnrInfoClient.getByUuid(info);
        if (infoResp == null || !infoResp.isSuccess() || infoResp.getData() == null || CuscStringUtils.isEmpty(
                infoResp.getData().getPhone())) {
            return Response.createError(ResponseCode.RNR_INFO_INVALID.getMsg(),
                    ResponseCode.RNR_INFO_INVALID.getCode());
        }
        info = infoResp.getData();

        //解绑
        VehicleUnbindDTO unbind = new VehicleUnbindDTO();
        unbind.setTenantNo(UserSubjectUtil.getTenantNo());
        unbind.setIsTrust(0);
        unbind.setIsSecondHandCar(0);
        info.setSerial_number(CuscStringUtils.generateUuid());
        info.setUuid(CuscStringUtils.generateUuid());
        info.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());

        unbind.setInfo(info);
        unbind.setOrder(createOrderInfo(info));
        unbind.setCardList(createCardInfo(info, unbind.getOrder().getUuid(), cardResp.getData().getIotId(), cardResp.getData().getIccid()));
        unbind.setCardIds(Arrays.asList(cardResp.getData().getUuid()));
        unbind.setRnrIds(Arrays.asList(cardResp.getData().getRnrId()));
        return unboundClient.batchUnbound(unbind);

    }

    private String getRnrIdByVinIccid(String vin, String iccid) {
        MgRnrCardInfoDTO bean = new MgRnrCardInfoDTO();
        bean.setIotId(vin);
        List<String> iccids = new ArrayList<>();
        iccids.add(iccid);
        bean.setIccidList(iccids);
        //bean.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<List<MgRnrCardInfoDTO>> mgRsp = mgRnrCardInfoClient.queryByList(bean);
        if (mgRsp.isSuccess()) {
            List<MgRnrCardInfoDTO> cardInfoDTOS = mgRsp.getData();
            if (!CollectionUtils.isEmpty(cardInfoDTOS)) {
                for (MgRnrCardInfoDTO cardInfo : cardInfoDTOS) {
                    if (null != cardInfo.getRnrStatus() && cardInfo.getRnrStatus().equals(RnrStatus.RNR.getCode())) {
                        return cardInfo.getRnrId();
                    }
                }
            }
        }
        return "";
    }


    //获取实名工单信息
    private RnrOrderDTO createOrderInfo(MgRnrInfoDTO info) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        rnrOrderDTO.setOrderType(RnrOrderType.UNBIND.getCode());
        rnrOrderDTO.setRnrId(info.getUuid());
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setTenantNo(info.getTenantNo());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.API.getCode());
        rnrOrderDTO.setSerialNumber(info.getSerial_number());
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.PASS.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setSendWorkOrder(false);
        rnrOrderDTO.setIsBatchOrder(IsBatchEnum.NO_BATCH.getCode());
        return rnrOrderDTO;
    }

    //获取车卡信息
    private List<MgRnrCardInfoDTO> createCardInfo(MgRnrInfoDTO info, String orderId, String iotId, String iccid) {
        List<MgRnrCardInfoDTO> retList = new ArrayList<>();
        MgRnrCardInfoDTO card = new MgRnrCardInfoDTO();
        card.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        card.setIccid(iccid);
        card.setIotId(iotId);
        card.setRnrId(info.getUuid());
        card.setOrderId(orderId);
        card.setCreator(UserSubjectUtil.getUserId());
        card.setSerial_number(info.getSerial_number());
        card.setUuid(CuscStringUtils.generateUuid());
        card.setTenantNo(info.getTenantNo());
        retList.add(card);
        return retList;
    }
}
