package com.cusc.nirvana.user.rnr.openapi.service.impl;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrUnbindClient;
import com.cusc.nirvana.user.rnr.fp.constants.BizTypeEnum;
import com.cusc.nirvana.user.rnr.fp.dto.FpRnrSmsInfoDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpSendMessageDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpVehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.fp.dto.SmsRequestDTO;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.rnr.openapi.constants.CustomerTypeEnum;
import com.cusc.nirvana.user.rnr.openapi.convert.PersonalRnrRequestConvert;
import com.cusc.nirvana.user.rnr.openapi.dto.PersonalRnrDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.PersonalRnrRequestDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.SecondHandUnbindRequestDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.VehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.openapi.service.ICardUnBindService;
import com.cusc.nirvana.user.rnr.openapi.service.IPersonalRnrService;
import com.cusc.nirvana.user.rnr.openapi.util.UserSubjectUtil;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 解绑实现类
 *
 * @author yubo
 * @since 2022-04-25 09:32
 */
@Service
@Slf4j
public class CardUnBindServiceImpl implements ICardUnBindService {

    @Autowired
    FpRnrUnbindClient rnrUnbindClient;

    @Resource
    private CardUnBindClient cardUnBindClient;

    @Autowired
    IPersonalRnrService personalRnrService;


    /**
     * 校验工单状态
     *
     * @param bean
     * @return
     */
    @Override
    public Response checkOrderStatus(RnrOrderDTO bean) {
        //短信校验
        Response response = cardUnBindClient.checkMessageOutTime(bean);
        if (!response.isSuccess()) {
            return response;
        }
        Response<RnrOrderDTO> orderStatus = cardUnBindClient.getOrderStatus(bean);
        if (!orderStatus.isSuccess()) {
            return Response.createError("获取工单信息失败", orderStatus.getData());
        }
        return orderStatus;
    }

    /**
     * 车主 解绑
     *
     * @param bean
     * @return
     */
    @Override
    public Response originalOwner(VehicleCardRnrDTO bean) {
        //log.info("自认人解绑入参: {}", JSON.toJSONString(bean));
        //校验
        FpVehicleCardRnrDTO fpVehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        fpVehicleCardRnrDTO.setVin(bean.getVin());
        Response<List<MgRnrCardInfoDTO>> cardList = cardUnBindClient.getCardList(fpVehicleCardRnrDTO);
        if (!cardList.isSuccess() || cardList.getData() == null) {
            //卡未实名或没有绑定在车上
            return Response.createError(cardList.getMsg(), cardList.getCode());
        }
        List<MgRnrCardInfoDTO> cardDatas = cardList.getData();
        List<String> iccidList = bean.getIccidList();
        List<MgRnrCardInfoDTO> rnrCardInfoDTOS = cardDatas.stream().filter(t -> iccidList.contains(t.getIccid())).collect(Collectors.toList());
        for (MgRnrCardInfoDTO rnrCardInfoDTO : rnrCardInfoDTOS) {
            rnrCardInfoDTO.setRnrStatus(RnrStatus.UNBOUND.getCode());
        }
        MgRnrInfoDTO rnrInfo = getRnrInfo(bean);
        if (rnrInfo == null) {
            return Response.createError("获取实名人员信息失败");
        }

        return this.sendMessage(bean);
    }


    /**
     * 二手车现车主解绑-自然人
     */
    @Override
    public Response secondHandPersonalUnbind(SecondHandUnbindRequestDTO bean) {
        //调用新车主实名认证接口
        PersonalRnrRequestDTO personalRnrRequestDTO = PersonalRnrRequestConvert.INSTANCE.secondHandUnbindDTOToPersonalRnrDTO(bean);
        personalRnrRequestDTO.setCustomerType(CustomerTypeEnum.USED_CAR_OWNER.getCode());
        personalRnrRequestDTO.setRnrBizzTypeEnum(RnrBizzTypeEnum.Unbound.getCode());
        Response<RnrRelationDTO> rnrRelationDTOResponse = personalRnrService.checkAndConvertToRelation(personalRnrRequestDTO, true);
        if (!rnrRelationDTOResponse.isSuccess()) {
            return Response.createError(rnrRelationDTOResponse.getMsg(), rnrRelationDTOResponse.getCode());
        }
        //调用fp接口
        RnrRelationDTO rnrRelationDTO = rnrRelationDTOResponse.getData();
        //修改order信息
        changeOrder(rnrRelationDTO.getOrder());
        Response rnrResponse = rnrUnbindClient.secondHandUnbind(rnrRelationDTO.getOrder().getSerialNumber(), bean.getRequestId(), rnrRelationDTO);
        if (rnrResponse.isSuccess()) {
            return Response.createSuccess(new PersonalRnrDTO((Integer) rnrResponse.getData()));
        }
        return rnrResponse;
    }

    //-----------------------私有方法区
    //修改order信息
    private void changeOrder(RnrOrderDTO order) {
        order.setAuditType(RnrOrderAuditTypeEnum.MANUAL.getCode());
        order.setAutoRnr(false);
        order.setOrderType(RnrOrderType.SEC_UNBIND.getCode());
        order.setSendWorkOrder(true);
    }

    private RnrOrderDTO createOrder(String rnrid, String tenNo, String orgId) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderType(RnrOrderType.UNBIND.getCode());
        rnrOrderDTO.setRnrId(rnrid);
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setIsBatchOrder(1);
        rnrOrderDTO.setSerialNumber(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.API.getCode());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setTenantNo(tenNo);
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.ASSIGNMENT.getCode());
        rnrOrderDTO.setOrgId(orgId);
        return rnrOrderDTO;
    }


    private MgRnrInfoDTO getRnrInfo(VehicleCardRnrDTO bean) {
        log.info("获取实名人员信息入参: {}", JSON.toJSONString(bean));
        //获取人员信息
        FpVehicleCardRnrDTO fpVehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        fpVehicleCardRnrDTO.setRnrId(bean.getRnrId());
        log.info("获取实名人员信息转换参数: {}", JSON.toJSONString(fpVehicleCardRnrDTO));
        Response<MgRnrInfoDTO> rnrInfo = cardUnBindClient.getRnrInfo(fpVehicleCardRnrDTO);
        if (!rnrInfo.isSuccess()) {
            return null;
        }
        return rnrInfo.getData();
    }

    /**
     * 发送短信
     *
     * @param bean
     * @return
     */
    @Override
    public Response sendMessage(VehicleCardRnrDTO bean) {
        log.info("自然人解绑短信发送入参: {}", JSON.toJSONString(bean));
        MgRnrInfoDTO rnrInfoData = getRnrInfo(bean);
        if (rnrInfoData == null) {
            return Response.createError("实名信息为空");
        }
        RnrOrderDTO order = this.createOrder(rnrInfoData.getUuid(), rnrInfoData.getTenantNo(), bean.getOrgId());
        //插入工单信息
        Response<RnrOrderDTO> rnrOrderDTOResponse = cardUnBindClient.insertOrder(order);
        if (!rnrOrderDTOResponse.isSuccess() || rnrOrderDTOResponse.getData() == null) {
            return Response.createError(rnrOrderDTOResponse.getMsg(), rnrOrderDTOResponse.getData());
        }
        //配置短信参数
        List<String> paramsList = new ArrayList<>();
        /*if (rnrInfoData.getGender() == 0) {
            paramsList.add(rnrInfoData.getFullName() + "女士");
        } else {
            paramsList.add(rnrInfoData.getFullName() + "先生");
        }*/
        paramsList.add(rnrInfoData.getFullName() + "女士/先生");
        paramsList.add(bean.getVin());

        //拼接iccid参数列表
        String iccIdList = "";
        for (String iccid : bean.getIccidList()) {
            iccIdList += iccid + ",";
        }
        String subIccIdList = iccIdList.substring(0, iccIdList.length() - 1);

        paramsList.add(subIccIdList);
        SmsRequestDTO smsRequestDTO = new SmsRequestDTO();
        smsRequestDTO.setBizType(BizTypeEnum.UNBOUND.getCode());
        smsRequestDTO.setParams(paramsList);
        smsRequestDTO.setPhone(rnrInfoData.getPhone());

        FpSendMessageDTO fpSendMessageDTO = new FpSendMessageDTO();
        fpSendMessageDTO.setRnrid(bean.getRnrId());
        fpSendMessageDTO.setUser(rnrInfoData.getCreator());
        fpSendMessageDTO.setIccidLists(subIccIdList);
        fpSendMessageDTO.setSmsRequestDTO(smsRequestDTO);
        fpSendMessageDTO.setOrderId(rnrOrderDTOResponse.getData().getUuid());

        //发送短信
        Response<FpRnrSmsInfoDTO> fpRnrSmsInfoDTOResponse = cardUnBindClient.sendSms(fpSendMessageDTO);

        if (!fpRnrSmsInfoDTOResponse.isSuccess() && fpRnrSmsInfoDTOResponse.getData() == null) {
            return Response.createError("短信发送失败", fpRnrSmsInfoDTOResponse.getCode());
        } else {
            return Response.createSuccess("短信发送成功", rnrOrderDTOResponse.getData());
        }
    }
}
