package com.cusc.nirvana.user.rnr.openapi.service.impl;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.ChangeTboxClient;
import com.cusc.nirvana.user.rnr.fp.dto.FpDirectTboxChangeDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpRnrSmsInfoDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FpVehicleCardRnrDTO;
import com.cusc.nirvana.user.rnr.fp.dto.UnbindReceiceSMSDTO;
import com.cusc.nirvana.user.rnr.mg.client.RnrOrderClient;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.ChangeTboxDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.IccidChangeDTO;
import com.cusc.nirvana.user.rnr.openapi.service.IChangeTboxService;
import com.cusc.nirvana.user.rnr.openapi.util.UserSubjectUtil;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;


/**
 * @author changh
 * @date 2022/4/24
 */
@Service
@Slf4j
public class ChangeTboxServiceImpl implements IChangeTboxService {

    @Resource
    private CardUnBindClient cardUnBindClient;

    @Resource
    private RnrOrderClient orderClient;

    @Resource
    private ChangeTboxClient changeTboxClient;


    @Override
    public Response submit(ChangeTboxDTO bean) {
        log.info("换卡入参：{}", JSON.toJSONString(bean));
        Map<String, String> newOldRelMap = new HashMap<>();
        for (IccidChangeDTO changeDTO : bean.getIccidList()) {
            newOldRelMap.put(changeDTO.getNewIccid(), changeDTO.getOldIccid());
        }

        //校验新的iccid
        Response<Boolean> checkStatus = changeTboxClient.checkStatus(new ArrayList<>(newOldRelMap.keySet()));
        log.info("校验新车卡出参：{}", JSON.toJSONString(checkStatus));
        if (!checkStatus.isSuccess() || !checkStatus.getData()) {
            return Response.createError("换卡iccid状态为已实名");
        }
        return this.changeTBox(bean, newOldRelMap);
    }


    private MgRnrInfoDTO getRnrInfo(ChangeTboxDTO bean) {
        //获取人员信息
        FpVehicleCardRnrDTO fpVehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        fpVehicleCardRnrDTO.setVin(bean.getVin());
        Response<List<MgRnrCardInfoDTO>> cardList = cardUnBindClient.getCardList(fpVehicleCardRnrDTO);
        if (!cardList.isSuccess() || CollectionUtils.isEmpty(cardList.getData())) {
            return null;
        }


        FpVehicleCardRnrDTO vehicleCardRnrDTO = new FpVehicleCardRnrDTO();
        vehicleCardRnrDTO.setRnrId(cardList.getData().get(0).getRnrId());
        Response<MgRnrInfoDTO> rnrInfo = cardUnBindClient.getRnrInfo(vehicleCardRnrDTO);
        if (!rnrInfo.isSuccess()) {
            return null;
        }
        return rnrInfo.getData();
    }

    private RnrRelationDTO turnParams(List<String> oldIccids, List<String> newIccids, String vin, String rnrid, String tenNo, String orgId) {
        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();
        String uuid = CuscStringUtils.generateUuid();
        RnrOrderDTO order = createOrder(rnrid, tenNo, orgId,uuid);
        List<MgRnrCardInfoDTO> cardInfoDTOS = new ArrayList<>();
        List<MgRnrCardInfoDTO> oldCardInfoDTOS = createCardInfoDTOS(oldIccids, tenNo, vin, rnrid, order.getUuid());
        List<MgRnrCardInfoDTO> newCardInfoDTOS = createCardInfoDTOS(newIccids, tenNo, vin, rnrid, order.getUuid());
        cardInfoDTOS.addAll(oldCardInfoDTOS);
        cardInfoDTOS.addAll(newCardInfoDTOS);
        rnrRelationDTO.setOrder(order);
        rnrRelationDTO.setCardList(cardInfoDTOS);
        return rnrRelationDTO;
    }

    private RnrOrderDTO createOrder(String rnrid, String tenNo, String orgId,String uuid) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(uuid);
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.ChangeBinding.getCode());
        rnrOrderDTO.setOrderType(RnrOrderType.TBOX_CHANGE.getCode());
        rnrOrderDTO.setRnrId(rnrid);
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setIsBatchOrder(1);
        rnrOrderDTO.setSerialNumber(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.API.getCode());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setTenantNo(tenNo);
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.TO_EXAMINE.getCode());
        rnrOrderDTO.setOrgId(orgId);
        return rnrOrderDTO;
    }

    private List<MgRnrCardInfoDTO> createCardInfoDTOS(List<String> iccidList, String tenNo, String vin, String rnrid, String uuid) {
        List<MgRnrCardInfoDTO> cardInfoDTOS = new ArrayList<>();
        for (String iccid : iccidList) {
            MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
            mgRnrCardInfoDTO.setUuid(CuscStringUtils.generateUuid());
            mgRnrCardInfoDTO.setIccid(iccid);
            mgRnrCardInfoDTO.setTenantNo(tenNo);
            mgRnrCardInfoDTO.setIotId(vin);
            mgRnrCardInfoDTO.setRnrId(rnrid);
            mgRnrCardInfoDTO.setOrderId(uuid);
            mgRnrCardInfoDTO.setCreator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setOperator(UserSubjectUtil.getUserId());
            mgRnrCardInfoDTO.setRnrBizzType(RnrBizzTypeEnum.ChangeBinding.getCode());
            cardInfoDTOS.add(mgRnrCardInfoDTO);
        }
        return cardInfoDTOS;
    }


    @Override
    public Response queryResult(ChangeTboxDTO bean) {
        //1.通过submit方法响应的新的rnrId查询业务结果
        return Response.createSuccess(true);
    }

    private Response changeTBox(ChangeTboxDTO bean, Map<String, String> newOldRelMap) {
        log.info("换卡入参：{}", JSON.toJSONString(bean));

        MgRnrInfoDTO rnrInfoData = getRnrInfo(bean);
        if (rnrInfoData == null) {
            return Response.createError("通过vin无法查询到此车实名信息");
        }

        String uuid = CuscStringUtils.generateUuid();

        RnrOrderDTO order = createOrder(rnrInfoData.getUuid(), rnrInfoData.getTenantNo(), bean.getOrgId(),uuid);

        //工单入库
        orderClient.add(order);

        //更改逻辑
        FpDirectTboxChangeDTO tboxChangeDTO = new FpDirectTboxChangeDTO();
        tboxChangeDTO.setNewOldIccidRelMap(newOldRelMap);
        tboxChangeDTO.setOperator(order.getOperator());
        tboxChangeDTO.setOrderUuid(order.getUuid());
        tboxChangeDTO.setRnrId(order.getRnrId());
        tboxChangeDTO.setRoutingKey(order.getRoutingKey());
        tboxChangeDTO.setTenantNo(order.getTenantNo());
        tboxChangeDTO.setVin(bean.getVin());
        Response changeRsp = changeTboxClient.directChangeBox(tboxChangeDTO);
        if (!changeRsp.isSuccess()) {
            return Response.createError(changeRsp.getMsg(), changeRsp.getData());
        }
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(uuid);
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.PASS.getCode());
        orderClient.updateOrderStatus(rnrOrderDTO);
        return Response.createSuccess("车机更换成功", changeRsp.getMsg());

    }

    //短信回调接口
    @Override
    public Response receiveMessage(UnbindReceiceSMSDTO bean) {
        log.info("换卡短信回调入参：{}", JSON.toJSONString(bean));
        return cardUnBindClient.receiveMessage(bean);
    }

    @Override
    public Response checkSMSStatus(FpRnrSmsInfoDTO bean) {
        log.info("换卡校验短信状态入参：{}", JSON.toJSONString(bean));
        Response<FpRnrSmsInfoDTO> messageStatus = cardUnBindClient.getMessageStatus(bean);
        if (!messageStatus.isSuccess() || messageStatus.getData() == null) {
            return Response.createError("获取短信失败");
        }
        long currentTimeMillis = System.currentTimeMillis();
        //校验时间2分钟内
        Date createTime = messageStatus.getData().getCreateTime();
        long startTime = createTime.getTime();
        if (currentTimeMillis - 120 * 1000 > startTime) {
            return Response.createError("短信获取超时");
        }
        //校验内容
        if (!messageStatus.getData().getReceiveContent().toLowerCase().equals("y")) {
            return Response.createError("短信回复为N");
        }
        return Response.createSuccess("短信校验通过");
    }
}
