package com.cusc.nirvana.user.rnr.openapi.service.impl;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.client.OrganizationClient;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.rnr.openapi.convert.OrganizationRespConvert;
import com.cusc.nirvana.user.rnr.openapi.dto.OrganizationRespDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.PagerDTO;
import com.cusc.nirvana.user.rnr.openapi.service.IOrganizationService;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class OrganizationServiceImpl implements IOrganizationService {

    @Autowired
    private OrganizationClient organizationClient;

    @Override
    public List<OrganizationRespDTO> getOrgListByPage(String tenantNo, PagerDTO pagerDTO) {
        List<OrganizationRespDTO> orgRespList = new ArrayList<>();

        if (tenantNo == null || tenantNo.equals(Strings.EMPTY)) {
            return orgRespList;
        }

        // 通过实名开放平台租户ID，获取组织结构
        OrganizationDTO dto = new OrganizationDTO();
        dto.setTenantNo(tenantNo);
        dto.setCurrPage(pagerDTO.getCurrPage());
        dto.setPageSize(pagerDTO.getPageSize());
        Response<PageResult<OrganizationDTO>> orgListResponse = organizationClient.queryByPage(dto);

        if (orgListResponse != null && orgListResponse.getData() != null) {
            orgRespList = orgListResponse.getData()
                    .getList()
                    .stream()
                    .map(ret -> OrganizationRespConvert.INSTANCE.convert(ret))
                    .collect(Collectors.toList());
        }

        return orgRespList;
    }
}
