package com.cusc.nirvana.user.rnr.openapi.service.impl;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.common.result.Response;

import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.FpRnrClient;
import com.cusc.nirvana.user.rnr.fp.client.VinCardClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrRelationClient;
import com.cusc.nirvana.user.rnr.mg.constants.*;
import com.cusc.nirvana.user.rnr.mg.dto.*;
import com.cusc.nirvana.user.rnr.openapi.constants.CustomerTypeEnum;
import com.cusc.nirvana.user.rnr.openapi.constants.IdentifyConstants;
import com.cusc.nirvana.user.rnr.openapi.convert.PersonalRnrRequestConvert;

import com.cusc.nirvana.user.rnr.openapi.dto.*;
import com.cusc.nirvana.user.rnr.openapi.service.IPersonalRnrService;
import com.cusc.nirvana.user.rnr.openapi.service.IVehicleCardService;
import com.cusc.nirvana.user.rnr.openapi.util.FileUtil;
import com.cusc.nirvana.user.rnr.openapi.util.SpringValidationUtil;
import com.cusc.nirvana.user.rnr.openapi.util.UserSubjectUtil;
import com.cusc.nirvana.user.rnr.openapi.util.ValidationUtil;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 个人实名信息
 *
 * @author **
 */
@Service
@Slf4j
public class PersonalRnrServiceImpl implements IPersonalRnrService {

    @Autowired
    IVehicleCardService vehicleCardService;

    @Autowired
    SmsServiceImpl smsService;

    @Autowired
    FpRnrClient fpRnrClient;

    @Autowired
    MgRnrRelationClient mgRnrRelationClient;

    @Autowired
    VinCardClient vinCardClient;

    @Autowired
    CardUnBindClient cardUnBindClient;


    /**
     * 检查参数并转换
     *
     * @param requestDTO
     * @return
     */
    @Override
    public Response<RnrRelationDTO> checkAndConvertToRelation(PersonalRnrRequestDTO requestDTO, boolean unbind) {
        log.info("Param checkAndConvertToRelation:{}", JSON.toJSONString(requestDTO));
        //参数必填校验
        Response response = validRnrInfo(requestDTO, unbind);

        if (!response.isSuccess()) {
            if (StringUtils.equals(response.getMsg(), ResponseCode.INVALID_DATA.getMsg())) {
                //返回第一个错误
                return Response.createError(((PersonalRnrResponseDTO) response.getData()).getResponseMsg().get(0), response.getCode());
            }
            return Response.createError(response.getMsg(), response.getCode());
        }

        return Response.createSuccess(convertToRelationDTO(requestDTO));
    }



    /**
     * 转成实名信息
     *
     * @param requestDTO 请求消息
     */
    private RnrRelationDTO convertToRelationDTO(PersonalRnrRequestDTO requestDTO) {
        String rnrID = CuscStringUtils.generateUuid();

        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();
        rnrRelationDTO.setRequestId(requestDTO.getRequestId());
        String tenantNo = UserSubjectUtil.getTenantNo();
        //租户
        rnrRelationDTO.setTenantNo(tenantNo);
        rnrRelationDTO.setIsTrust(requestDTO.getIsConsigner() ? 1 : 0);
        rnrRelationDTO.setIsSecondHandCar(null != requestDTO.getCustomerType() && requestDTO.getCustomerType() == 1 ? 1 : 0);
        //实名实体信息
        rnrRelationDTO.setInfo(getMgRnrInfoDTO(requestDTO, rnrID));
        //实名工单信息
        rnrRelationDTO.setOrder(getMgRnrOrderDTO(requestDTO, rnrRelationDTO));
        rnrRelationDTO.getOrder().setTenantNo(tenantNo);
        //实名卡信息
        rnrRelationDTO.setCardList(getMgRnrCardDTOs(requestDTO, rnrRelationDTO));
        //联系人信息
        if (requestDTO.getIsConsigner()) {
            rnrRelationDTO.setRnrLiaisonList(getMgRnrLiaisonDTOs(requestDTO, rnrRelationDTO));
        }
        //实名文件信息
        rnrRelationDTO.setRnrFileList(getMgRnrFileDTOs(requestDTO, rnrRelationDTO));

        rnrRelationDTO.setSerialNumber(requestDTO.getSerialNumber());
        return rnrRelationDTO;
    }


    /**
     * 创建工单信息
     *
     * @param requestDTO     请求消息
     * @param rnrRelationDTO 实名消息
     */
    private RnrOrderDTO getMgRnrOrderDTO(PersonalRnrRequestDTO requestDTO, RnrRelationDTO rnrRelationDTO) {
        //二手车、或者证书类型不是身份证都是手工
        //boolean sendWorkOrder = isSendWorkOrder(requestDTO);
        boolean isSecondHandCar = rnrRelationDTO.getIsSecondHandCar() == 1;

        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setOrderType(isSecondHandCar ? RnrOrderType.SEC_VEHICLE.getCode() : RnrOrderType.NEW_VEHICLE.getCode());
        rnrOrderDTO.setRnrId(rnrRelationDTO.getInfo().getUuid());
        //rnrOrderDTO.setAuditType(sendWorkOrder ? RnrOrderAuditTypeEnum.MANUAL.getCode() : RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setIsBatchOrder(0);
        rnrOrderDTO.setSerialNumber(rnrRelationDTO.getInfo().getSerial_number());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.API.getCode());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setSendWorkOrder(false);
        //rnrOrderDTO.setOrderStatus(sendWorkOrder ? RnrOrderStatusEnum.ASSIGNMENT.getCode() : RnrOrderStatusEnum.PASS.getCode());
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.PASS.getCode());
        rnrOrderDTO.setRnrBizzType(requestDTO.getRnrBizzTypeEnum());
        rnrOrderDTO.setOrgId(requestDTO.getOrgId());
        return rnrOrderDTO;
    }

    /**
     * 判断是否是是否需要发送工单
     *
     * @param requestDTO
     * @return
     */
//    private boolean isSendWorkOrder(PersonalRnrRequestDTO requestDTO) {
//        //二手车实名，委托人模式，新车车主非身份证模式都需要发送工单
//        return requestDTO.getCustomerType() == CustomerTypeEnum.USED_CAR_OWNER.getCode()
//                || requestDTO.getIsConsigner()
//                || !StringUtils.equalsIgnoreCase(requestDTO.getCertType(), CertTypeEnum.IDCARD.getCode());
//    }

    /**
     * 获取联系人信息
     *
     * @param requestDTO 请求消息
     */
    private List<MgRnrLiaisonInfoDTO> getMgRnrLiaisonDTOs(PersonalRnrRequestDTO requestDTO, RnrRelationDTO rnrRelationDTO) {
        List<MgRnrLiaisonInfoDTO> liaisonInfoDTOS = new ArrayList<>();
        ConsignerInfoDTO consignerInfo = requestDTO.getConsignerInfo();
        MgRnrLiaisonInfoDTO mgRnrLiaisonInfoDTO = new MgRnrLiaisonInfoDTO();
        mgRnrLiaisonInfoDTO.setLiaisonName(consignerInfo.getFullName());
        mgRnrLiaisonInfoDTO.setLiaisonPhone(consignerInfo.getPhone());
        mgRnrLiaisonInfoDTO.setLiaisonCertAddress(consignerInfo.getCertAddress());
        mgRnrLiaisonInfoDTO.setLiaisonCertNumber(consignerInfo.getCertNumber());
        mgRnrLiaisonInfoDTO.setLiaisonCertType(consignerInfo.getCertType());
        mgRnrLiaisonInfoDTO.setLiaisonContactAddress(consignerInfo.getContactAddress());
        mgRnrLiaisonInfoDTO.setLiaisonType(RnrLiaisonType.CONSIGNEE.getCode());
        mgRnrLiaisonInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        mgRnrLiaisonInfoDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrLiaisonInfoDTO.setRnrId(rnrRelationDTO.getInfo().getUuid());
        mgRnrLiaisonInfoDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrLiaisonInfoDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrLiaisonInfoDTO.setRnrBizzType(requestDTO.getRnrBizzTypeEnum());
        mgRnrLiaisonInfoDTO.setLiaisonGender(StringUtils.isEmpty(consignerInfo.getGender())?null:Integer.valueOf(consignerInfo.getGender()));
        mgRnrLiaisonInfoDTO.setLiaisonExpiredDate(consignerInfo.getCertExpirationDate());
        liaisonInfoDTOS.add(mgRnrLiaisonInfoDTO);
        return liaisonInfoDTOS;
    }

    /**
     * 获取实名文件信息
     *
     * @param requestDTO 请求消息
     */
    private List<MgRnrFileDTO> getMgRnrFileDTOs(PersonalRnrRequestDTO requestDTO, RnrRelationDTO rnrRelationDTO) {
        List<MgRnrFileDTO> mgRnrFileDTOS = new ArrayList<>();

        String rnrId = rnrRelationDTO.getInfo().getUuid();

        int rnrBizzTypeEnum = requestDTO.getRnrBizzTypeEnum();

        //证件照片
        List<String> certPic = requestDTO.getCertPic();
        for (int i = 0; i < certPic.size(); i++) {
            String pic = certPic.get(i);
            Integer fileType = FileUtil.getFileType(requestDTO.getCertType(), i);
            if (fileType != null) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, pic, fileType, rnrBizzTypeEnum));
            }
        }

        if (requestDTO.getIsConsigner()) {
            String consignerId = rnrRelationDTO.getRnrLiaisonList().get(0).getUuid();
            //委托人照片
            List<String> consignerPic = requestDTO.getConsignerInfo().getCertPic();
            for (int i = 0; i < consignerPic.size(); i++) {
                String pic = consignerPic.get(i);
                Integer fileType = FileUtil.getFileType(requestDTO.getConsignerInfo().getCertType(), i);
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, pic, fileType, consignerId, rnrBizzTypeEnum));
            }
            //委托书
            for (String attorney : requestDTO.getConsignerInfo().getAttorneyLetterPic()) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, attorney, RnrFileType.LETTER_ATTORNEY.getCode(), consignerId, rnrBizzTypeEnum));
            }
            //活体视频
            if (StringUtils.isNotBlank(requestDTO.getLiveVerificationVideo())) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, requestDTO.getLiveVerificationVideo(), RnrFileType.LIVENESS_VIDEO.getCode(), consignerId, rnrBizzTypeEnum));
            }
        } else {
            //活体视频
            if (StringUtils.isNotBlank(requestDTO.getLiveVerificationVideo())) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, requestDTO.getLiveVerificationVideo(), RnrFileType.LIVENESS_VIDEO.getCode(), rnrBizzTypeEnum));
            }
        }

        //二手车，购车合同，购车发票，过户合同
        if (null != requestDTO.getCustomerType() && requestDTO.getCustomerType() == CustomerTypeEnum.USED_CAR_OWNER.getCode()) {
            if (!CollectionUtils.isEmpty(requestDTO.getPurchaseInvoicePic())) {
                for (String invoice : requestDTO.getPurchaseInvoicePic()) {
                    mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, invoice, RnrFileType.CAR_PURCHASE_INVOICE.getCode(), rnrBizzTypeEnum));
                }
            }

            for (String contract : requestDTO.getPurchaseContractPic()) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, contract, RnrFileType.CAR_PURCHASE_CONTRACT.getCode(), rnrBizzTypeEnum));
            }

            for (String transfer : requestDTO.getTransferCertificatePic()) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, transfer, RnrFileType.CAR_TRANSFER_CERTIFICATE.getCode(), rnrBizzTypeEnum));
            }
        }

        //入网合同
        if (!CollectionUtils.isEmpty(requestDTO.getContractPic())) {
            for (String contractPic : requestDTO.getContractPic()) {
                mgRnrFileDTOS.add(FileUtil.newFileDTO(rnrId, contractPic, RnrFileType.VEHUCLE_BIND.getCode(), rnrBizzTypeEnum));
            }
        }

        return mgRnrFileDTOS;
    }

    /**
     * 获取实名卡信息
     *
     * @param requestDTO 请求消息
     */
    private List<MgRnrCardInfoDTO> getMgRnrCardDTOs(PersonalRnrRequestDTO requestDTO, RnrRelationDTO rnrRelationDTO) {
        List<String> iccidList = requestDTO.getIccidList();
        List<MgRnrCardInfoDTO> mgRnrCardInfoDTOS = new ArrayList<>();
        MgRnrCardInfoDTO mgRnrCardInfoDTO;
        if (!CollectionUtils.isEmpty(iccidList)) {
            for (String iccid : iccidList) {
                mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
                mgRnrCardInfoDTO.setUuid(CuscStringUtils.generateUuid());
                mgRnrCardInfoDTO.setIccid(iccid);
                mgRnrCardInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
                mgRnrCardInfoDTO.setIotId(requestDTO.getVin());
                mgRnrCardInfoDTO.setRnrId(rnrRelationDTO.getInfo().getUuid());
                mgRnrCardInfoDTO.setOrderId(rnrRelationDTO.getOrder().getUuid());
                mgRnrCardInfoDTO.setCreator(UserSubjectUtil.getUserId());
                mgRnrCardInfoDTO.setOperator(UserSubjectUtil.getUserId());
                mgRnrCardInfoDTO.setRnrBizzType(requestDTO.getRnrBizzTypeEnum());
                mgRnrCardInfoDTOS.add(mgRnrCardInfoDTO);
            }
        }

        return mgRnrCardInfoDTOS;
    }

    /**
     * 获取实名主体信息
     *
     * @param requestDTO 请求消息
     */
    private MgRnrInfoDTO getMgRnrInfoDTO(PersonalRnrRequestDTO requestDTO, String rnrID) {
        MgRnrInfoDTO mgRnrInfoDTO = PersonalRnrRequestConvert.INSTANCE.requestDTOToMgRnrDTO(requestDTO);
        mgRnrInfoDTO.setIsCompany(0);
        mgRnrInfoDTO.setIsTrust(requestDTO.getIsConsigner() ? 1 : 0);
        mgRnrInfoDTO.setIsSecondHandCar(Objects.equals(requestDTO.getCustomerType(), CustomerTypeEnum.NEW_CAR_OWNER.getCode()) ? 0 : 1);
        mgRnrInfoDTO.setUuid(rnrID);
        mgRnrInfoDTO.setCreator(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setOperator(UserSubjectUtil.getUserId());
        mgRnrInfoDTO.setSerial_number(CuscStringUtils.generateUuid());
        mgRnrInfoDTO.setTenantNo(UserSubjectUtil.getTenantNo());
        mgRnrInfoDTO.setRnrStatus(requestDTO.getRnrStatus());
        mgRnrInfoDTO.setRnrBizzType(requestDTO.getRnrBizzTypeEnum());
        mgRnrInfoDTO.setExpiredDate(requestDTO.getCertExpirationDate());
        return mgRnrInfoDTO;
    }

    /**
     * 进行参数验证
     *
     * @param requestDTO 请求消息
     */
    private Response validRnrInfo(PersonalRnrRequestDTO requestDTO, boolean unbind) {
//        if (StringUtils.isBlank(requestDTO.getPurchaseContract())) {
//            return Response.createError("购车合同不能为空", ResponseCode.INVALID_DATA.getCode());
//        }
//        if (StringUtils.isBlank(requestDTO.getPurchaseInvoice())) {
//            return Response.createError("购车证明不能为空", ResponseCode.INVALID_DATA.getCode());
//        }
        //检查手机
        Response response = ValidationUtil.checkPhone(requestDTO.getPhone());
        if (!response.isSuccess()) {
            return response;
        }

        //性别校验
        response = validGender(requestDTO.getGender());
        if (!response.isSuccess()) {
            return response;
        }

        //校验证件号码
        response = ValidationUtil.checkIdentifyNumber(requestDTO.getCertType(), requestDTO.getCertNumber());
        if (!response.isSuccess()) {
            return response;
        }

        //证件照数量验证
        response = ValidationUtil.checkIdentifyPics(requestDTO.getCertType(), requestDTO.getCertPic().size());
        if (!response.isSuccess()) {
            return response;
        }

        //二手车车主，需要有过户证明
        if (requestDTO.getCustomerType() == CustomerTypeEnum.USED_CAR_OWNER.getCode()) {
            if (CollectionUtils.isEmpty(requestDTO.getTransferCertificatePic())) {
                return Response.createError("过户证明不能为空", ResponseCode.INVALID_DATA.getCode());
            }
            if (CollectionUtils.isEmpty(requestDTO.getPurchaseContractPic())) {
                return Response.createError("购车合同不能为空", ResponseCode.INVALID_DATA.getCode());
            }
            if (!unbind && CollectionUtils.isEmpty(requestDTO.getPurchaseInvoicePic())) {
                return Response.createError("购车发票不能为空", ResponseCode.INVALID_DATA.getCode());
            }
        }

        if (requestDTO.getIsConsigner()) {
            //验证委托人关系
            ConsignerInfoDTO consignerInfo = requestDTO.getConsignerInfo();

            Set<ConstraintViolation<ConsignerInfoDTO>> violations = SpringValidationUtil.groupVerificationParameters(consignerInfo);
            if (!violations.isEmpty()) {
                return Response.createError(violations.stream().findFirst().get().getMessage());
            }

            //性别校验
            response = validGender(consignerInfo.getGender());
            if (!response.isSuccess()) {
                return response;
            }

            //校验证件号码
            response = ValidationUtil.checkIdentifyNumber(consignerInfo.getCertType(), consignerInfo.getCertNumber());
            if (!response.isSuccess()) {
                return response;
            }

            //证件照数量验证
            response = ValidationUtil.checkIdentifyPics(consignerInfo.getCertType(), consignerInfo.getCertPic().size());
            if (!response.isSuccess()) {
                return response;
            }

            //检查手机
            response = ValidationUtil.checkPhone(consignerInfo.getPhone());
            if (!response.isSuccess()) {
                return response;
            }
        }
        //api层不需要验证码
        // else {
        //没有委托人，车主验证码必填
//            if (StringUtils.isBlank(requestDTO.getVerificationCode())) {
//                return Response.createError("手机验证码不能为空", ResponseCode.INVALID_DATA.getCode());
//            }
        //}

        return Response.createSuccess();
    }

    /**
     * 校验性别
     *
     * @param gender
     */
    private Response validGender(String gender) {
        if (!gender.equals(IdentifyConstants.GENDER_MALE)
                && !gender.equals(IdentifyConstants.GENDER_FEMALE)) {
            return Response.createError("性别格式错误", ResponseCode.INVALID_DATA.getCode());
        }
        return Response.createSuccess();
    }

}
