package com.cusc.nirvana.user.rnr.openapi.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.client.VinCardClient;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.openapi.service.IVehicleCardService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author yubo
 * @since 2022-04-18 09:26
 */
@Service
@Slf4j
public class VehicleCardServiceImpl implements IVehicleCardService {


    @Autowired
    VinCardClient vinCardClient;

    @Autowired
    CardUnBindClient cardUnBindClient;

    @Value("${fileTemplate.manufacturer.fileSize: 500}")
    private String fileSize;

    @Resource
    private FileSystemClient fileSystemClient;

    @Resource
    private MgRnrCardInfoClient cardInfoClient;


    /**
     * 车卡关系和卡校验
     *
     * @param vinCardDTO
     * @return
     */
    @Override
    public Response<VerifyVinCardResponseDTO> verifyVinCard(@Valid com.cusc.nirvana.user.rnr.openapi.dto.VinCardInfoDTO vinCardDTO) {
        //调用批量验证接口
        List<VinCardDTO> vinCardDTOS
                = vinCardDTO.getIccidList().stream().map(iccid -> new VinCardDTO(vinCardDTO.getVin(), iccid)).collect(Collectors.toList());

        return verifyVinCardBatch(vinCardDTOS);
    }

    /**
     * 批量车卡关系和卡校验
     *
     * @param verifyDTOs
     * @return
     */
    @Override
    public Response<VerifyVinCardResponseDTO> verifyVinCardBatch(List<VinCardDTO> verifyDTOs) {
        //进行vin和iccid的验证
        Response<VerifyVinCardResponseDTO> response = vinCardClient.verifyVinCard(verifyDTOs);
        if (!response.isSuccess()) {
            return response;
        }

        return Response.createSuccess(response.getData());
    }



    @Override
    public Response<List<MgRnrCardInfoDTO>> getBindsByVinsAndIccids(List<String> vins, List<String> iccidList) {
        MgRnrCardInfoDTO bean = new MgRnrCardInfoDTO();
        bean.setVinList(vins);
        bean.setIccidList(iccidList);
        Response<List<MgRnrCardInfoDTO>> cardListResponse = cardInfoClient.queryBindListByVinsAndIccids(bean);
        if (!cardListResponse.isSuccess()) {
            return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
        }
        return cardListResponse;
    }

}
